#!/bin/sh

#########################################################################
# Author:       Tony Green
# Date:         23/08/04
# Script name:  raid-check
# Script desc:  Detect failed RAID arrays
#########################################################################

#########################################################################
# Change these variables when porting to a new machine
#########################################################################

# Arrary device (sccli)
ARRAYDEV='/dev/rdsk/c6t600C0FF0000000000053BC0AB943D000d0s2'

# Controller addresses (from luxadm)
CNTL1='216000c0ff8053bc'
CNTL2='210000e08b13f17d'


#########################################################################
# End of modification section
#########################################################################

# Declare-a-go-go
METASTAT=/usr/sbin/metastat
EGREP=/bin/egrep
SCCLI=/usr/sbin/sccli
LUXADM=/usr/sbin/luxadm
LINE="\n##################################################################\n"


if [ "$1" = '-v' ]
then
	VERBOSE=1
fi

# Check meta devices (software RAID on the system volumes)
METASTAT=`$METASTAT  | $EGREP "State:"  | $EGREP -cv "State: Okay"`

if [ -n "$VERBOSE" -a "$METASTAT" -gt 0 ]
then
	echo $LINE
	echo "METASTAT Errors"
	echo "---------------"
	$METASTAT | $EGREP "State:" | $EGREP -v "State: Okay"
fi

$SCCLI $ARRAYDEV show logical-drives 2>/dev/null 1>/dev/null || exit 99


# Check the logical drives on the 3510
LD=`$SCCLI $ARRAYDEV show logical-drives 2>/dev/null  | $EGREP "^ld[0-9]"  | $EGREP -vc "Good"`
if [ -n "$VERBOSE" -a "$LD" -gt 0 ]
then
	echo $LINE
	echo "Logical Drive Errors"
	echo "--------------------\n"
	$SCCLI $ARRAYDEV show logical-drives 2>/dev/null  | $EGREP "^ld[0-9]"  | $EGREP -v "Good"
fi



# Check the physical drives on the 3510
DISKS=`$SCCLI $ARRAYDEV show disks 2>/dev/null | $EGREP -v "ONLINE|STAND-BY|S/N|IDs|-----" | $EGREP -vc "ONLINE"`
if [ -n "$VERBOSE" -a "$DISKS" -gt 0 ]
then
	echo $LINE
	echo "Physical Drive Errors"
	echo "---------------------\n"
	$SCCLI $ARRAYDEV show disks 2>/dev/null | $EGREP -v "ONLINE|STAND-BY|S/N|IDs|-----" | $EGREP -v "ONLINE"
fi



# Check the redundancy on the 3510
REDUNDANCY=`$SCCLI $ARRAYDEV show redundancy-mode 2>/dev/null | $EGREP "Redundancy" | $EGREP -vc "Active-Active|Enabled"`
if [ -n "$VERBOSE" -a "$REDUNDANCY" -gt 0 ]
then
	echo $LINE
	echo "Redundancy Errors"
	echo "------------------\n"
	$SCCLI $ARRAYDEV show redundancy-mode 2>/dev/null | $EGREP "Redundancy" | $EGREP -v "Active-Active|Enabled"
fi

# Check the event log on the 3510
EVENT=`$SCCLI $ARRAYDEV show events 2>/dev/null | $EGREP -v "battery" | $EGREP -ic "ALERT"`
if [ -n "$VERBOSE" -a "$EVENT" -gt 0 ]
then
	echo $LINE
	echo "Event Errors"
	echo "------------\n"
	$SCCLI $ARRAYDEV show events 2>/dev/null | $EGREP -i "ALERT"
fi


# Check the status of the fibre links
LUX=`$LUXADM disp $ARRAYDEV 2>/dev/null | $EGREP "State"  | $EGREP -vc "ONLINE"`
if [ -n "$VERBOSE" -a "$LUX" -gt 0 ]
then
	echo $LINE
	echo "Fibre Errors"
	echo "------------\n"
	$LUXADM disp $ARRAYDEV 2>/dev/null | $EGREP "State"  | $EGREP -vc "ONLINE"
fi


# Check the status of the controllers
CNTL=`$LUXADM disp $ARRAYDEV 2>/dev/null | $EGREP -c "$CNTL1|$CNTL2"`

# If there's not two found (the two we know about), then barf
if [ "$CNTL" != 2 ]
then
	CNTL=99
else
	CNTL=0
fi

if [ -n "$VERBOSE" -a "$CNTL" -gt 0 ]
then
	echo $LINE
	echo "Controller Errors"
	echo "-----------------\n"
	$LUXADM disp $ARRAYDEV 2>/dev/null | $EGREP -c "$CNTL1|$CNTL2"
fi


# Sum up all the failures and...
COUNT=`expr $METASTAT + $LD + $DISKS + $REDUNDANCY + $EVENT + $LUX + $CNTL`

if [ -n "$VERBOSE" -a "$COUNT" -gt 0 ]
then
	echo $LINE
	echo "Total Errors : $COUNT\n"
fi

# .... exit with the total number of problems
#echo $COUNT
exit $COUNT
